<?php
/**
*
* functions_postnotices.php
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}


/**
*	This function does the following:
*	-	turns any quote or code bbcode tag into a simple plain text lead-in to or
*		exit from the quote or code block (if the post has a quote tag that specifies
*		a particular person that is being quoted, the code adds at the beginning the plain text
*		words "written by John Brown" or whoever the quote is from),
*
*	-	turns the [*] bbocode (marking a list item) into a simple plain text asterisk,
*
*	-	turns a url or img with an alias into a simple plain text url or img with brackets around it and
*
*	-	strips out all other bbcode.
*
*	Thanks to Merlin Sythove for helping me (asinshesq) come up with this code
*/
function bbcode_process_to_plain_text($text, $bbcode_uid = '', $lang)
{
	// First, add a colon before bbcode_uid since all bbcode_uid identifiers are always preceded by a colon
	$bbcode_uid = ':' . $bbcode_uid;

	// Next, remove slashes
	$text = html_entity_decode(trim($text));
	// Before we start with bbcode removal, convert html links or email addresses to plain text
	$text = preg_replace('/<!-- w --><a class="postlink" href="(.*?)">(.*?)<\/a><!-- w -->/', '\\1', $text);
	$text = preg_replace('/<!-- m --><a class="postlink" href="(.*?)">(.*?)<\/a><!-- m -->/', '\\1', $text);
	$text = preg_replace('/<!-- e --><a href="mailto:(.*?)">(.*?)<\/a><!-- e -->/', '\\1', $text);
	// and clean up smilies ;)
	$text = preg_replace('/<!-- s(.*?) --><img src="(.*?)" alt="(.*?)" title="(.*?)" \/><!-- s(.*?) -->/', '[\\4]', $text);

	// Now, we define the plain text markers used for [ quote ] and [ code ] blocks, list items, and [ img ] open and close tags
	$newline = "\r\n";
	$q1 = $newline . '>>>>>>>>>>>>>>>>>>>>>>' . $newline; // quote or code opening marker
	$q2 = $newline . '<<<<<<<<<<<<<<<<<<<<<<' . $newline; $newline; // quote or code closing marker
	$l1 = $newline . '   (*) '; // list item marker
	$img1 = '[' . $lang['IMAGE_AT'] . ': '; // img opening marker
	$img2 = ' ]'; // img closing marker

	// Next, replace open and close quote tags with plain text versions
    $text = preg_replace('/\[quote=\"(.*?)\"' . $bbcode_uid . '\]/', $newline . '\\1 '. $lang['WROTE'].':'.$q1, $text);
	$text = preg_replace('/\[quote' . $bbcode_uid . '\]/', $newline . $lang['QUOTE'].':'.$q1, $text);
	$text = preg_replace('/\[\/quote' . $bbcode_uid . '\]/', $q2, $text);

	// And now, replace code tags with plain text versions
	$text = preg_replace('/\[code' . $bbcode_uid . '\]/', $newline . $lang['CODE'].':'.$q1, $text);
	$text = preg_replace('/\[\/code' . $bbcode_uid . '\]/', $q2, $text);

	// Now, replace any open or close brackets that have been converted by phpbb into [ or ] (e.g if nested inside of code tags)
	$text = preg_replace('/&#91;/', '[', $text);
	$text = preg_replace('/&#93;/', ']', $text);

	// Next, change list item bbcode ([*]) into simple asterisks (note: does not process list numbers or letters)
	$text = preg_replace('/\[\*' . $bbcode_uid . '\]/', $l1, $text);

	// Now, process image and url bbcode (get rid of aliases and insert the actual image or url address; put images inside of square prackets).
	$text = preg_replace('/\[img' . $bbcode_uid . '\](.*?)\[\/img' . $bbcode_uid . '\]/', $img1 .'\\1' . $img2, $text);
	$text = preg_replace('/\[url=(.*?)' . $bbcode_uid . '\](.*?)\[\/url' . $bbcode_uid . '\]/', '\\2 ' . '[ \\1 ]', $text);
	$text = preg_replace('/\[url' . $bbcode_uid . '\](.*?)\[\/url' . $bbcode_uid . '\]/', '\\1', $text);

	// Now, remove all bracketed expressions with bbcode_uid identifier in them
	$text = preg_replace('/\[([^[]*?)' . $bbcode_uid . '([^]]*?)\]/', '', $text);

	// And finally, change multiple blank lines into one blank line
	$text = preg_replace('/(\\r\\n){3,}/s', $newline.$newline, $text);

	return $text;
}


/**
* put together the subject and body text for the notice
*/
function compile_notice_text(&$postnotices_data, $username, $mode, $forum_id, $topic_id, $post_id, $recipient, $recipient_lang, $posttext_array)
{
	global $config, $db, $phpEx, $phpbb_root_path, $user;
	if (!isset($postnotices_data[$recipient_lang]))
	{
		$username = ($mode == 'edit') ? $username : $user->data['username'];
		file_exists($phpbb_root_path . 'language/' . $recipient_lang . '/mods/postnotices.' . $phpEx) ? include($phpbb_root_path . 'language/' . $recipient_lang . '/mods/postnotices.' . $phpEx) : include($phpbb_root_path . 'language/en/mods/postnotices.' . $phpEx);
		$postnotices_data[$recipient_lang]['subject']  = ($postnotices_data['postnotices_subject']) ? htmlspecialchars_decode($postnotices_data['postnotices_subject']) : sprintf($lang['POSTNOTICES_SUBJECT_DEFAULT'], $config['sitename']);
		$postnotices_data[$recipient_lang]['message']  = ($postnotices_data['postnotices_message']) ? htmlspecialchars_decode($postnotices_data['postnotices_message']) : sprintf($lang['POSTNOTICES_MESSAGE_DEFAULT'], $username, $config['sitename']);
		$postnotices_data[$recipient_lang]['message'] .= ($posttext_array) ? sprintf($lang['POSTNOTICES_POSTTEXT'], bbcode_process_to_plain_text($posttext_array['post_text'], $posttext_array['bbcode_uid'], $lang)) : '';
		$postnotices_data[$recipient_lang]['message'] .= sprintf($lang['POSTNOTICES_LINK'], generate_board_url() . "/viewtopic.$phpEx?f=$forum_id&t=$topic_id&p=$post_id#p$post_id");
		$postnotices_data[$recipient_lang]['message'] .= ($postnotices_data['postnotices_board_signs']) ? str_replace('<br />', "\n", "-- \n" . $config['board_email_sig']) : "-- \n" . htmlspecialchars_decode($user->data['username']);
	}
}


/**
* draws the form in the posting page that allows user to
* select groups that will receive notice of a post or reply, specify subject matter and message, etc.
*/
function draw_postnotices_form($postnotices_data, $username, $forum_id)
{
	global $auth, $config, $db, $template, $user;

	// return without drawing the form if user is not allowed to send postnotices to groups or the post needs to be approved
	if (!$auth->acl_get('f_postnotices', $forum_id) || (!$auth->acl_get('f_noapprove', $forum_id) && !$auth->acl_get('m_approve', $forum_id)))
	{
		return;
	}

	$user->add_lang('mods/postnotices');

	// define special groups like BOTS that should not show up in the list
	$skipped_groups = array('BOTS', 'GUESTS', 'REGISTERED_COPPA');

	// now look up all groups other than the skipped groups or hidden groups that the user does not have permission to know about
	if ($auth->acl_gets('a_group', 'a_groupadd', 'a_groupdel'))
	{
		$sql = 'SELECT group_id, group_name, group_type
			FROM ' . GROUPS_TABLE . '
			WHERE ' . $db->sql_in_set('group_name', $skipped_groups, true) . '
			ORDER BY group_name';
	}
	else
	{
		$sql = 'SELECT DISTINCT g.group_id, g.group_name, g.group_type
			FROM ' . GROUPS_TABLE . ' g, ' . USER_GROUP_TABLE . ' ug
			WHERE ' . $db->sql_in_set('g.group_name', $skipped_groups, true) . '
				AND
				(
					g.group_type <> ' . GROUP_HIDDEN . '
					OR
					(
						g.group_id = ug.group_id
						AND ug.user_id = ' . $user->data['user_id'] . '
						AND ug.user_pending = 0
					)
				)
			ORDER BY g.group_name';
	}
	$result = $db->sql_query($sql);

	while ( $row = $db->sql_fetchrow($result) )
	{
		$template->assign_block_vars('postnoticeslistrow', array(
		'S_POSTNOTICES_GROUP_CHECK'	=> ($postnotices_data && in_array($row['group_id'], $postnotices_data['postnotices_group_ids'])) ? 'checked="checked"' : '',
		'S_POSTNOTICES_GROUP_ID'		=> $row['group_id'],
		'U_POSTNOTICES_GROUP_NAME'	=> ($row['group_type'] != GROUP_SPECIAL) ? $row['group_name'] : $user->lang['G_' . $row['group_name']])
		);
	}

	$db->sql_freeresult($result);

	$template->assign_vars(array(
		'POSTNOTICES_MESSAGE_DEFAULT'	=> sprintf($user->lang['POSTNOTICES_MESSAGE_DEFAULT'], $username, $config['sitename']),
		'POSTNOTICES_MESSAGE_EXPLAIN'	=> $user->lang['POSTNOTICES_MESSAGE_EXPLAIN'],
		'POSTNOTICES_SUBJECT_DEFAULT'	=> sprintf($user->lang['POSTNOTICES_SUBJECT_DEFAULT'], $config['sitename']),
		'POSTNOTICES_SUBJECT_EXPLAIN'	=> $user->lang['POSTNOTICES_SUBJECT_EXPLAIN'],
		'S_SHOW_POSTNOTICES'			=> true,
		));

	if ($postnotices_data)
	{
		// this is a preview, so put in what user has inputted
		$template->assign_vars(array(
			'POSTNOTICES_MESSAGE'					=> $postnotices_data['postnotices_message'],
			'POSTNOTICES_SUBJECT'					=> $postnotices_data['postnotices_subject'],
			'S_POSTNOTICES_BOARD_SIGNS_CHECK'		=> (!empty($postnotices_data['postnotices_board_signs'])) ? 'checked="checked"' : '',
			'S_POSTNOTICES_I_SIGN_CHECK'			=> (empty($postnotices_data['postnotices_board_signs'])) ? 'checked="checked"' : '',
			'S_POSTNOTICES_INCLUDE_POSTTEXT_CHECK'	=> (!empty($postnotices_data['postnotices_include_posttext'])) ? 'checked="checked"' : '',
			)
		);
	}
	else
	{
		// this is the first pass, so put in the defaults
		$template->assign_vars(array(
			'S_POSTNOTICES_BOARD_SIGNS_CHECK'		=> 'checked="checked"',
			'S_POSTNOTICES_I_SIGN_CHECK'			=> '',
			'S_POSTNOTICES_INCLUDE_POSTTEXT_CHECK'	=> '',
			)
		);
	}
}


/**
* look up the post_text and bbcode_uid
*/
function fetch_posttext($post_id)
{
	global $db;

	// user is including post text in notice, so fetch bbcode_uid for the post and run text through
	// bbcode_process_to_plain_text() to clean up any bbcode in it
	$sql = 'SELECT post_text, bbcode_uid
		FROM ' . POSTS_TABLE . '
		WHERE post_id = ' . $post_id;
	$result = $db->sql_query($sql);
	$row = $db->sql_fetchrow($result);

	return array('post_text' => $row['post_text'], 'bbcode_uid' => $row['bbcode_uid']);
}



/**
* get postnotices to groups info the user has filled out in the form (used for preview and when the actual notices are sent)
*/
function get_postnotices_data_from_form()
{
	$postnotices_data = array();
	$postnotices_data['postnotices_group_ids']			= request_var('postnotices_group_ids', array(0));
	$postnotices_data['postnotices_subject']			= utf8_normalize_nfc(request_var('postnotices_subject', '', true));
	$postnotices_data['postnotices_message']			= utf8_normalize_nfc(request_var('postnotices_message', '', true));
	$postnotices_data['postnotices_include_posttext']	= (!empty($_POST['postnotices_include_posttext'])) ? true : false;
	$postnotices_data['postnotices_board_signs']		= (!empty($_POST['postnotices_board_signs'])) ? true : false;
	return $postnotices_data;
}


/**
* include the specified groups in the list of users who will get notice of the post or reply
*/
function include_groups_as_recipients($forum_id, &$notify_rows, &$sql_ignore_users, &$postnotices_data)
{
	global $auth, $db, $user;

	if (!$auth->acl_get('f_postnotices', $forum_id))
	{
		return;
	}

	// fetch info the user inputted into the form
	$postnotices_data = get_postnotices_data_from_form();

	// turn group_id array into comma separated list for use in query and later on for use in storage...
	// Note: would have used $db->sql_in_set on the array in the query except that we need a comma separated list
	// to store in posts table later on and there's no point in doing two implodes
	$group_id_list = implode (', ', $postnotices_data['postnotices_group_ids']);

	// now lets find the users who should be getting notified (it will test for read authorization later in user_notification())
	$sql = 'SELECT DISTINCT u.user_id, u.username, u.user_email, u.user_lang, u.user_notify_type, u.user_jabber
		FROM ' . USERS_TABLE . ' u, ' . USER_GROUP_TABLE . ' ug
		WHERE ug.group_id IN (' . $group_id_list . ')
			AND u.user_id = ug.user_id
			AND (' . $db->sql_in_set('u.user_id', $sql_ignore_users, true) . ' OR u.user_id = ' . $user->data['user_id'] . ')
			AND ug.user_pending = 0
			AND u.user_type IN (' . USER_NORMAL . ', ' . USER_FOUNDER . ')';
	$result = $db->sql_query($sql);

	$notify_rows = array();
	while ($row = $db->sql_fetchrow($result))
	{
		$notify_user_id = (int) $row['user_id'];
		$notify_rows[$notify_user_id] = array(
			'user_id'		=> $row['user_id'],
			'username'		=> $row['username'],
			'user_email'	=> $row['user_email'],
			'user_jabber'	=> $row['user_jabber'],
			'user_lang'		=> $row['user_lang'],
			'notify_type'	=> 'postnotices',
			'template'		=> 'postnotices',
			'method'		=> $row['user_notify_type'],
			'allowed'		=> false
		);
		
		// make sure the subsequent topic and forum notices don't go out to people already receiving postnotices
		$sql_ignore_users[$notify_user_id] = $notify_user_id;
	}
	$db->sql_freeresult($result);

	return $group_id_list;
}


/**
* update the posts table to record notices sent to groups
*/
function record_postnotices($group_id_list, $post_id, $postnotices)
{
	global $db;
	if ($postnotices)
	{
		// this is an edit of a prior post that has already been sent out to some groups, so create a cumulative non-duplicative list
		$current_list_array = explode(', ', $group_id_list);
		$old_list_array = explode(', ', $postnotices);
		$combined_list_array = array_merge($current_list_array, array_diff($old_list_array,$current_list_array));
		$group_id_list = implode(', ', $combined_list_array);
	}

	// store the group_ids in the db for later retrieval in viewtopic (each post that has been emailed will include a list of recipient groups visible to anyone who is authorized to email)
	$sql = 'UPDATE ' . POSTS_TABLE . '
		SET postnotices = "' . $group_id_list . '"
		WHERE post_id = ' . $post_id;
	$db->sql_query($sql);
}


/**
* determine whether this user should have the option to send notices to groups for this post or reply
*/
function show_postnotices_option($forum_id)
{
	global $auth, $config;

	// show option only if user is allowed to send postnotices to groups and the post does not need to be approved
	$show_postnotices_option = (!$auth->acl_get('f_postnotices', $forum_id) || (!$auth->acl_get('f_noapprove', $forum_id) && !$auth->acl_get('m_approve', $forum_id))) ? false : true;
	return $show_postnotices_option;
}


?>